mod AST;
pub mod encoder;
pub mod parser;
mod tests;

use encoder::CodeFormat;
use encoder::SymbolTable;

use parser::Section;

use crate::loader::unloader::make_string;

use self::parser::SectionContent;

impl Section {
    fn get_size(&self) -> usize {
        match &self.content {
            SectionContent::Code(c) => c.len(),
            SectionContent::CString(s) => {
                let c = s.len();
                if c % 2 != 0 {
                    (c + 1) / 2 + 1
                } else {
                    c / 2 + 1
                }
            }
            SectionContent::CVec() => todo!(),
        }
    }

    fn to_binary(&self, sy: &SymbolTable) -> Option<Vec<u16>> {
        let own_address = sy.lookup(&self.name);
        match &self.content {
            SectionContent::Code(c) => {
                let mut res = vec![];
                let mut pc = own_address;
                for op in c.iter() {
                    println!("converting {:?}", op);
                    res.push(CodeFormat::encode_op(op, sy, pc)?);
                    pc += 1;
                }
                return Some(res);
            }
            SectionContent::CString(s) => {
                return Some(make_string(s));
            }
            SectionContent::CVec() => todo!(),
        }
    }
}

fn sort_sections(sections: Vec<Section>) -> Option<Vec<Section>> {
    // we start with a mock section that we'll just replace.
    let mut res: Vec<Section> = vec![Section {
        name: "".to_owned(),
        content: SectionContent::CString("".to_owned()),
    }];

    let mut nocode: Vec<Section> = vec![];

    for section in sections.into_iter() {
        // trick: if let to pattern match discarding result
        if let SectionContent::Code(_) = section.content {
            if section.name == "main" {
                res[0] = section;
            } else {
                res.push(section);
            }
        } else {
            nocode.push(section);
        }
    }

    res.append(&mut nocode);

    return Some(res);
}

fn make_symbol_table<'a>(sections: &'a Vec<Section>) -> Option<SymbolTable> {
    let mut res = vec![];
    let mut pos: u16 = 0;

    for sec in sections {
        res.push((sec.name.clone(), pos));
        pos += sec.get_size() as u16;
    }

    return Some(SymbolTable(res));
}

pub fn to_binary(sections: Vec<Section>) -> Option<Vec<u16>> {
    let sorted = sort_sections(sections)?;
    println!("sorted sections: {:?}", sorted);
    let sy = make_symbol_table(&sorted)?;
    println!("symbol table: {:?}", sy);
    let k: Vec<Vec<u16>> = sorted
        .iter()
        .map(|x| x.to_binary(&sy))
        .collect::<Option<Vec<Vec<u16>>>>()?;
    println!("binary sections: {:?}", k);

    return Some(k.into_iter().flatten().collect());
}
